from django.contrib import admin

from timed_auth_token.models import TimedAuthToken


@admin.register(TimedAuthToken)
class TokenAdmin(admin.ModelAdmin):
    list_display = ('key', 'user', 'created', 'last_used', 'expires',)
    fields = ('user',)
    ordering = ('-last_used',)



import os
import base64

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
from django.utils import timezone

from .settings import token_settings


class TimedAuthToken(models.Model):
    '''An auth token that expires.

    The token duration is specified on the model class as a timedelta
    named `token_validity_duration`.

    If you use the included TimedAuthTokenAuthentication then the
    expiration date is refreshed every time the token is used.

    Example:
        class MyUser(models.Model):
            token_validity_duration = timedelta(days=30)

        token = TimedAuthToken(user=MyUser())

    '''
    key = models.CharField(max_length=40, primary_key=True)
    created = models.DateTimeField(auto_now_add=True)
    last_used = models.DateTimeField(auto_now=True)
    expires = models.DateTimeField(blank=True)

    user = models.ForeignKey(settings.AUTH_USER_MODEL)

    def __str__(self):
        return 'Token for %s, created %s, expires %s' % (
            self.user.get_username(), self.created, self.expires
        )

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()

        if not self.expires:
            self.calculate_new_expiration()

        super(TimedAuthToken, self).save(*args, **kwargs)

    def calculate_new_expiration(self):
        validity_duration = getattr(get_user_model(), 'token_validity_duration',
                                    token_settings.DEFAULT_VALIDITY_DURATION)
        self.expires = timezone.now() + validity_duration

    @property
    def is_expired(self):
        return self.expires < timezone.now()

    @staticmethod
    def generate_key():
        return base64.urlsafe_b64encode(os.urandom(30)).decode()



from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers, exceptions

from .models import TimedAuthToken


class TimedAuthTokenCreateSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()

    def validate_username(self, username):
        try:
            User = get_user_model()
            self.user = User.objects.get(**{User.USERNAME_FIELD: username})
        except ObjectDoesNotExist:
            msg = _('That {} does not exist.'.format(User.USERNAME_FIELD))
            raise serializers.ValidationError(msg)

    def validate(self, data):
        password = data['password']
        if not self.user.check_password(password):
            raise serializers.ValidationError(_('Incorrect password.'))

        if not self.user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        return data

    def create(self, validated_data):
        return TimedAuthToken.objects.create(user=self.user)

    def to_representation(self, token):
        return TimedAuthTokenReadSerializer(token).data

    def update(self, instance, validated_data):
        raise RuntimeError('Cannot update a timed auth token.')


class TimedAuthTokenReadSerializer(serializers.ModelSerializer):
    class Meta:
        model = TimedAuthToken
        fields = 'token',

    token = serializers.CharField(source='key')



from datetime import timedelta

from django.conf import settings


DEFAULT = {
    'DEFAULT_VALIDITY_DURATION': timedelta(days=30)
}


class TokenSettings:
    def __getattr__(self, key):
        try:
            return settings.TIMED_AUTH_TOKEN[key]
        except AttributeError:
            return DEFAULT[key]

    def __getitem__(self, key):
        return self.__getattr__(key)


token_settings = TokenSettings()
